package com.jiuqi.script.rhino;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Callable;
import org.mozilla.javascript.Scriptable;
import com.jiuqi.script.IScriptObject;
import org.mozilla.javascript.ScriptableObject;

public class RhinoScriptObject extends ScriptableObject
{
    public static final String CLASSNAME = "CustomScriptObject";
    private IScriptObject sobj;
    
    public RhinoScriptObject(final IScriptObject sobj) {
        this.sobj = sobj;
    }
    
    public String getClassName() {
        return "CustomScriptObject";
    }
    
    public IScriptObject getScriptObject() {
        return this.sobj;
    }
    
    public Object[] getAllIds() {
        return this.getIds();
    }
    
    public Object[] getIds() {
        final String[] properties = this.sobj.getPropertyNames();
        final String[] functions = this.sobj.getFunctionNames();
        final String[] ids = new String[properties.length + functions.length];
        System.arraycopy(properties, 0, ids, 0, properties.length);
        System.arraycopy(functions, 0, ids, properties.length, functions.length);
        return ids;
    }
    
    public Object get(final String name, final Scriptable start) {
        if (this.sobj.hasFunction(name)) {
            return new ScriptFunction(name, this.sobj);
        }
        return this.sobj.getProperty(name);
    }
    
    public Object get(final int index, final Scriptable start) {
        final String[] properties = this.sobj.getPropertyNames();
        if (properties.length > index) {
            return this.sobj.getProperty(this.sobj.getPropertyNames()[index]);
        }
        return null;
    }
    
    public boolean has(final String name, final Scriptable start) {
        return this.sobj.hasProperty(name);
    }
    
    public boolean has(final int index, final Scriptable start) {
        final String[] properties = this.sobj.getPropertyNames();
        return properties.length > index && this.sobj.hasProperty(this.sobj.getPropertyNames()[index]);
    }
    
    public void put(final String name, final Scriptable start, final Object value) {
        this.sobj.setProperty(name, value);
    }
    
    public void delete(final String name) {
        this.sobj.delProperty(name);
    }
    
    public void delete(final int index) {
        final String[] properties = this.sobj.getPropertyNames();
        if (properties.length > index) {
            this.sobj.delProperty(this.sobj.getPropertyNames()[index]);
        }
    }
    
    private class ScriptFunction implements Callable
    {
        private String functionName;
        private IScriptObject scriptObject;
        
        public ScriptFunction(final String functionName, final IScriptObject scriptObject) {
            this.functionName = functionName;
            this.scriptObject = scriptObject;
        }
        
        public Object call(final Context cx, final Scriptable scope, final Scriptable thisObj, final Object[] args) {
            if (args == null) {
                return this.scriptObject.invoke(this.functionName, new Object[0]);
            }
            final Object[] newArgs = new Object[args.length];
            for (int i = 0; i < args.length; ++i) {
                newArgs[i] = ScriptHelper._toJava(args[i], scope);
            }
            return this.scriptObject.invoke(this.functionName, newArgs);
        }
    }
}
